//
//  CustomCountDownView.m
//  MobNewsSDKApp
//
//  Created by 兵伍 on 2020/2/20.
//  Copyright © 2020 兵伍. All rights reserved.
//

#import "CustomCountDownView.h"

typedef NS_ENUM(NSInteger, CustomCountDownViewStatus) {
    CustomCountDownViewStatusInit,
    CustomCountDownViewStatusRunning,
    CustomCountDownViewStatusPaused,
    CustomCountDownViewStatusEnd
};

#define CUSTOM_COUNT_DOWN_LINE_WIDTH 5

@interface CustomCountDownView()
@property (nonatomic, strong) CAShapeLayer *lineLayer;
@property (nonatomic, strong) CAShapeLayer *railLayer;
@property (nonatomic, strong) NSTimer *timer;

@property (nonatomic, assign) NSInteger rewardNum;
@property (nonatomic, assign) NSInteger duration;
@property (nonatomic, strong) UILabel *timeLabel;
@property (nonatomic, strong) UIImageView *iconImgView;
@property (nonatomic, assign) CustomCountDownViewStatus status;
@end

@implementation CustomCountDownView

#pragma mark - Lifecycle

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        self.backgroundColor = [UIColor whiteColor];
    }
    return self;
}

- (void)dealloc {
    NSLog(@"%s", __func__);
}

- (void)layoutSubviews {
    [super layoutSubviews];
    self.iconImgView.center = self.timeLabel.center;
}

- (void)touchesEnded:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [super touchesEnded:touches withEvent:event];
//    if ([[BMNUtils sharedUtil].delegate respondsToSelector:@selector(lk_adSDKDidClickCountDownView)]) {
//        [[BMNUtils sharedUtil].delegate lk_adSDKDidClickCountDownView];
//    }
}

#pragma mark - Functions

- (void)startWithDuration:(NSInteger)duration frame:(CGRect)rect scrollDelay:(CGFloat)delay {
    if (self.status == CustomCountDownViewStatusInit || self.status == CustomCountDownViewStatusEnd) {
        self.scrollDelay = delay;
        [self.delegate countDownViewDidStart:self];
        
        self.frame = rect;
        
        CGFloat sWidth = self.bounds.size.width;
        CGFloat sHeight = self.bounds.size.height;
        
        self.layer.cornerRadius = sHeight / 2.0;
        self.railLayer.hidden = NO;
        
        CGFloat w = (self.bounds.size.width - 9 * 2);
        self.timeLabel.frame = CGRectMake(0, 0, w, w);
        self.timeLabel.center = CGPointMake(sWidth / 2.0, sHeight / 2.0);
        self.timeLabel.layer.cornerRadius = w / 2.0;
        
        // 添加阴影
        CGFloat shadowOffset = 2;
        CGFloat radius = rect.size.width / 2.0;
        CALayer *layer1 = self.layer;
        CGRect rect = self.bounds;
        rect.origin.y = 2;
        layer1.shadowPath=[UIBezierPath bezierPathWithRoundedRect:CGRectInset(rect, -shadowOffset, -shadowOffset) byRoundingCorners:UIRectCornerAllCorners cornerRadii:CGSizeMake(radius + shadowOffset / 2.0, radius + shadowOffset / 2.0)].CGPath;
        layer1.shadowColor = [UIColor colorWithWhite:200/255.0 alpha:1].CGColor;
        layer1.shadowOffset = CGSizeMake(0, 0);
        layer1.shadowRadius = 4;
        layer1.shadowOpacity = 1;
        
        // 开始
        self.duration = duration;
    }
}

- (void)pause {
    if (self.status == CustomCountDownViewStatusPaused) {
        return;
    }
    self.status = CustomCountDownViewStatusPaused;
    CFTimeInterval pauseTime = [self.lineLayer convertTime:CACurrentMediaTime() fromLayer:nil];
    self.lineLayer.timeOffset = pauseTime;
    self.lineLayer.speed = 0;
    
    [self.timer setFireDate:[NSDate distantFuture]];
}

- (void)resume {
    
    if (self.status == CustomCountDownViewStatusRunning) {
        return;
    }
    self.status = CustomCountDownViewStatusRunning;
    //CABasicAnimation *ani = [self.lineLayer animationForKey:@"countDown"];
    //[self.lineLayer addAnimation:ani forKey:@"countDown"];
    
    CFTimeInterval pausedTime = self.lineLayer.timeOffset;
    self.lineLayer.speed = 1.0;
    self.lineLayer.timeOffset = 0.0;
    self.lineLayer.beginTime = 0.0;
    CFTimeInterval timeSincePause = [self.lineLayer convertTime:CACurrentMediaTime() fromLayer:nil] - pausedTime;
    self.lineLayer.beginTime = timeSincePause;
    
    [self.timer setFireDate:[NSDate distantPast]];

}

- (void)showRewardWithInfo:(NSString *)info {
    self.timeLabel.text = info;
    
#if 1
    if (!([info isKindOfClass:[NSString class]] && info.length > 0)) {
        //[self removeFromSuperview];
        return;
    }
    UILabel *l = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, self.bounds.size.width, 0)];
    l.textColor = [UIColor redColor];
    l.font = [UIFont systemFontOfSize:22];
    l.text = info;
    [l sizeToFit];
    l.transform = CGAffineTransformMakeScale(0.3, 0.3);

    l.alpha = 0.3;
    l.center = CGPointMake(self.bounds.size.width / 2.0, -10);
    [self addSubview:l];
    
    [UIView animateWithDuration:1 delay:0 options:UIViewAnimationOptionCurveEaseInOut animations:^{
        l.alpha = 1;
        l.transform = CGAffineTransformIdentity;
        l.center = CGPointMake(self.bounds.size.width / 2.0, -34);
    } completion:^(BOOL finished) {
        [self delayRemove];
    }];
    
    CAKeyframeAnimation * keyAnimaion1 = [CAKeyframeAnimation animation];
    keyAnimaion1.keyPath = @"transform.rotation";
    keyAnimaion1.values = @[@(-20 / 180.0 * M_PI),@(20 /180.0 * M_PI),@(-20/ 180.0 * M_PI),@(0 /180.0 * M_PI)];//度数转弧度
    keyAnimaion1.removedOnCompletion = NO;
    keyAnimaion1.fillMode = kCAFillModeForwards;
    //keyAnimaion1.duration = 0.5;
    //keyAnimaion1.repeatCount = 3;//动画次数
    
    CAKeyframeAnimation * keyAnimaion2 = [CAKeyframeAnimation animation];
    keyAnimaion2.keyPath = @"transform.scale";
    keyAnimaion2.values = @[@(1.2),@(0.9),@(1.1),@(0.95),@(1)];//度数转弧度
    keyAnimaion2.removedOnCompletion = NO;
    keyAnimaion2.fillMode = kCAFillModeForwards;
    //keyAnimaion2.duration = 0.5;
    //keyAnimaion2.repeatCount = 3;//动画次数
    
    CAAnimationGroup *group = [[CAAnimationGroup alloc] init];
    group.animations = @[keyAnimaion1, keyAnimaion2];
    group.repeatCount = 2;
    group.duration = 0.4;
    
    [self.iconImgView.layer addAnimation:group forKey:nil];
#endif
}

- (void)delayRemove {
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [self removeFromSuperview];
    });
}

- (void)tick:(NSTimer *)timer {
    _duration--;
    if (_duration <= 0) {
        _duration = 0;
        self.timeLabel.text = [NSString stringWithFormat:@"%zds", (NSInteger)_duration];

        [self.timer invalidate];
        self.timer = nil;
        self.status = CustomCountDownViewStatusEnd;
        if ([self.delegate respondsToSelector:@selector(countDownViewDidTimeout:)]) {
            [self.delegate countDownViewDidTimeout:self];
        }
        
        return;
    }
    self.timeLabel.text = [NSString stringWithFormat:@"%zds", (NSInteger)_duration];
}

#pragma mark - Getter and Setter

- (void)setDuration:(NSInteger)duration {
    _duration = duration;
    self.timeLabel.text = [NSString stringWithFormat:@"%zds", (NSInteger)_duration];
    
    UIBezierPath *path = [UIBezierPath bezierPathWithArcCenter:CGPointMake(self.lineLayer.bounds.size.width/2.0, self.lineLayer.bounds.size.height/2.0) radius:self.lineLayer.bounds.size.width/2.0 - CUSTOM_COUNT_DOWN_LINE_WIDTH / 2.0 startAngle:-0.5 * M_PI endAngle:1.5 * M_PI clockwise:YES];
    self.lineLayer.path = path.CGPath;
    
    // 绘图动画
    CABasicAnimation *pathAnimation = [CABasicAnimation animationWithKeyPath:@"strokeEnd"];// strokeStart 是擦除效果
    pathAnimation.duration = _duration;
    pathAnimation.fromValue = [NSNumber numberWithFloat:0.0f];
    pathAnimation.toValue = [NSNumber numberWithFloat:1.0f];
    pathAnimation.removedOnCompletion = NO;
    
    [_lineLayer removeAnimationForKey:@"countDown"];
    [_lineLayer addAnimation:pathAnimation forKey:@"countDown"];
    
    
    [self.timer invalidate];
    self.timer = nil;
    self.timer = [NSTimer timerWithTimeInterval:1 target:self selector:@selector(tick:) userInfo:nil repeats:YES];
    // 如果需要避免滑动时影响，使用NSRunLoopCommonModes
    [[NSRunLoop currentRunLoop]  addTimer:self.timer forMode:NSRunLoopCommonModes];
    //[[NSRunLoop currentRunLoop] run];
    self.status = CustomCountDownViewStatusRunning;

}

- (UIImageView *)iconImgView {
    if (!_iconImgView) {
        _iconImgView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 28, 34)];
        _iconImgView.image = [UIImage imageNamed:@"countDown_center_icon"];
        [self addSubview:_iconImgView];
    }
    return _iconImgView;
}

- (UILabel *)timeLabel {
    if (!_timeLabel) {
        UILabel *label = [[UILabel alloc] initWithFrame:self.bounds];
        label.textAlignment = NSTextAlignmentCenter;
        label.clipsToBounds = YES;
        label.textColor = [UIColor redColor];
        label.backgroundColor = [UIColor whiteColor];
        [self addSubview:label];
        _timeLabel = label;
    }
    return _timeLabel;
}

- (CAShapeLayer *)railLayer {
    if (!_railLayer) {
        _railLayer = [CAShapeLayer layer];
        _railLayer.frame = self.lineLayer.frame;
        _railLayer.backgroundColor = [UIColor clearColor].CGColor;
        _railLayer.strokeColor = [UIColor colorWithWhite:245/255.0 alpha:1].CGColor;
        _railLayer.fillColor = nil;
        _railLayer.lineWidth = CUSTOM_COUNT_DOWN_LINE_WIDTH;
        _railLayer.lineJoin = kCALineJoinRound;
        [self.layer insertSublayer:_railLayer below:self.lineLayer];
        
        UIBezierPath *path = [UIBezierPath bezierPathWithArcCenter:CGPointMake(self.lineLayer.bounds.size.width/2.0, self.lineLayer.bounds.size.height/2.0) radius:self.lineLayer.bounds.size.width/2.0 - CUSTOM_COUNT_DOWN_LINE_WIDTH / 2.0 startAngle:-0.5 * M_PI endAngle:1.5 * M_PI clockwise:YES];
        _railLayer.path = path.CGPath;
        _railLayer.strokeStart = 0;
        _railLayer.strokeEnd = 1;
    }
    return _lineLayer;
}

- (CAShapeLayer *)lineLayer {
    if (!_lineLayer) {
        _lineLayer = [CAShapeLayer layer];
        CGRect rect = CGRectInset(self.bounds, 5, 5);
        _lineLayer.frame = rect;
        _lineLayer.backgroundColor = [UIColor clearColor].CGColor;
        _lineLayer.strokeColor = [UIColor redColor].CGColor;
        _lineLayer.fillColor = nil;
        _lineLayer.lineWidth = CUSTOM_COUNT_DOWN_LINE_WIDTH;
        _lineLayer.lineCap = kCALineCapRound;
        [self.layer addSublayer:_lineLayer];
    }
    return _lineLayer;
}

@end
